clc,clear,close all

% 
% Illustration script: optimal sampling of 3D decaying exponentials.
%
%
rng(2*pi)

N1 = 50; % nbr of available samples, first dimension
N2 = 100; % nbr of available samples, first dimension
N3 = 70; % nbr of available samples, first dimension
K = 200; % nbr of desired samples
t1_vec = (0:N1-1)'; % time vector, first dimension
t2_vec = (0:N2-1)'; % time vector, second dimension
t3_vec = (0:N3-1)'; % time vector, second dimension

nbr_exponentials = 2; % nbr of components 

%%%%%% Choose parameters for the components %%%%%%%
amp_vec = 3+2*(rand(1,nbr_exponentials)-.5);

beta1_vec = 5e-2 + 5e-2*rand(1,nbr_exponentials);
beta2_vec = 5e-2 + 5e-2*rand(1,nbr_exponentials);
beta3_vec = 5e-2 + 5e-2*rand(1,nbr_exponentials);
freq1_vec = .4*rand(1,nbr_exponentials);
freq2_vec = .4*rand(1,nbr_exponentials);
freq3_vec = .4*rand(1,nbr_exponentials);
phi_vec = 2*pi*randn(1,nbr_exponentials);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


sigma2 = 10; % noise variance

P = 8*nbr_exponentials; % nbr of model parameters parametrar (amp,beta1,beta2,beta3,f1,f2,f3,phi)

gap_tol = 1e-5; % tolerance for duality gap. Used as convergence criterion.

% Fisher matrices
% This is given to the solver as a cell of length N, where N = N1*N2*N3. Each
% Fisher matrix corresponds to a sample, i.e., a time (t_1,t_2,t_3). The
% sampling scheme will select such pairs, so we have to keep track of in
% which order we put the into the cell.
parameterorder = 'by_parameter';
N = N1*N2*N3;
F_cell1 = cell(N,1);
for k_time1 = 1:N1
    t1temp = t1_vec(k_time1);
    for k_time2 = 1:N2
        t2temp = t2_vec(k_time2);
        for k_time3 = 1:N3  
            temp_index = k_time3+(k_time1-1)*N2*N3+(k_time2-1)*N3;
            t3temp = t3_vec(k_time3);
            [fisherMat,paramOrder] = getFisher3D...
                (t1temp,t2temp,t3temp,amp_vec,freq1_vec,freq2_vec,freq3_vec,...
                beta1_vec,beta2_vec,beta3_vec,phi_vec...
                ,sigma2,parameterorder);
            F_cell1{temp_index} = fisherMat;
        end
    end
end


%% Put weight only on decay and frequency parameters.

psiVec1 = [zeros(nbr_exponentials,1);...
    ones(nbr_exponentials,1);ones(nbr_exponentials,1);ones(nbr_exponentials,1);...
    ones(nbr_exponentials,1);ones(nbr_exponentials,1);ones(nbr_exponentials,1);...
    zeros(nbr_exponentials,1)];
%%
% Solve sampling problem
[w,wInd,terminationFlag_1,argsOut_1] = ...
    sampling_scheme_selector(F_cell1,K,psiVec1,gap_tol);

%% Plot sample scheme
%
% w is the vector of sample weights. Reshape to N3\times N2\times N1 tensor (this
% was the order we entered them in F_cell1).
%

W = reshape(w,N3,N2,N1);
[I3,I2,I1] = ind2sub([N3,N2,N1],wInd);

plot3(I1,I2,I3,'ko','MarkerFaceColor','k','markersize',5)
grid on
xlabel('t_1'),ylabel('t_2'),zlabel('t_3')

%%
figure(2)
%%%%%%%%%%%%
if length(argsOut_1.primalObjectiveHist)>30
    start_plot_iter = 10;
    iter_inds = start_plot_iter:length(argsOut_1.primalObjectiveHist);
else
    start_plot_iter = 1;
    iter_inds = start_plot_iter:length(argsOut_1.primalObjectiveHist);
end
plot(iter_inds,argsOut_1.primalObjectiveHist(iter_inds),'linewidth',1.5)
hold on
plot(iter_inds,argsOut_1.dualObjectiveHist(iter_inds),'--','linewidth',1.5)
hold off
grid on
xlabel('iteration'),xlim([-Inf,Inf])
legend('primal objective','dual objective','location','best')
title('Algorithm convergence')
%%%%%%%%%%%%%

%%
